# -*- coding: utf-8 -*-
"""
Created on Thu Jan 27 22:34:12 2022

@author: Will

A library of materials to be used by modelling functions 

"""
def custom_dielectric(mws, name, epsilon, mu, tanD, thermalConductivity):
    material = mws.Material
    material.Reset()
    material.name(name)
    material.FrqType('all')
    material.Type('Normal')
    material.MaterialUnit('Frequency', 'GHz')
    material.MaterialUnit('Geometry', 'mm')
    material.epsilon(str(epsilon))
    material.Mu(str(mu))
    material.Kappa('0.0')
    material.tanD(str(tanD))
    material.tanDFreq('10.0')
    material.tanDGiven('True')
    material.tanDModel('ConsttanD')
    material.KappaM('0.0')
    material.tanDM('0.0')
    material.tanDMFreq('0.0')
    material.tanDMGiven('False')
    material.tanDMModel('ConstKappa')
    material.DispModelEps('None')
    material.DispModelMu('None')
    material.DispersiveFittingSchemeEps('General 1st')
    material.DispersiveFittingSchemeMu('General 1st')
    material.UseGeneralDispersionEps('False')
    material.UseGeneralDispersionMu('False')
    material.Rho('0.0')
    material.ThermalType('Normal')
    material.thermalConductivity(str(thermalConductivity))
    material.ThermalType('Normal')
    material.SetActiveMaterial('all')
    material.Colour('0.94', '0.82', '0.76')
    material.Wireframe('False')
    material.Transparency('0')
    material.Create()
    format(material)

def air(mws):
    material = mws.Material
    material.Reset()
    material.Name('Air')
    material.FrqType('all')
    material.Type('Normal')
    material.MaterialUnit('Frequency', 'GHz')
    material.MaterialUnit('Geometry', 'mm')
    material.Epsilon('1.00059')
    material.Mu('1.0')
    material.Kappa('0.0')
    material.TanD('0.0')
    material.TanDFreq('0.0')
    material.TanDGiven('False')
    material.TanDModel('ConstKappa')
    material.KappaM('0')
    material.TanDM('0.0')
    material.TanDMFreq('0.0')
    material.TanDMGiven('False')
    material.TanDMModel('ConstKappa')
    material.DispModelEps('None')
    material.DispModelMu('None')
    material.DispersiveFittingSchemeEps('General 1st')
    material.DispersiveFittingSchemeMu('General 1st')
    material.UseGeneralDispersionEps('False')
    material.UseGeneralDispersionMu('False')
    material.Rho('1.204')
    material.ThermalType('Normal')
    material.ThermalConductivity('0.026')
    material.HeatCapacity('1.005')
    material.SetActiveMaterial('all')
    material.Colour('0.682353', '0.717647', '1')
    material.Wireframe('False')
    material.Transparency('0')
    material.Create()
    format(material)
    
def copper_annealed_lossy(mws):
    material = mws.Material
    material.Reset()
    material.Name('Copper (annealed)')
    material.FrqType('static')
    material.Type('Normal')
    material.SetMaterialUnit('Hz', 'mm')
    material.Epsilon('1')
    material.Mue('1.0')
    material.Kappa('5.8e+007')
    material.TanD('0')
    material.TanDFreq('0.0')
    material.TanDGiven('False')
    material.TanDModel('ConstTanD')
    material.KappaM('0.0')
    material.TanDM('0.0')
    material.TanDMFreq('0.0')
    material.TanDMGiven('False')
    material.TanDMModel('ConstTanD')
    material.DispModelEps('None')
    material.DispModelMue('None')
    material.DispersiveFittingSchemeEps('Nth Order')
    material.DispersiveFittingSchemeMue('Nth Order')
    material.UseGeneralDispersionEps('False')
    material.UseGeneralDispersionMue('False')
    material.FrqType('all')
    material.Type('Lossy metal')
    material.SetMaterialUnit('GHz', 'mm')
    material.Mue('1.0')
    material.Kappa('5.8e+007')
    material.Rho('8930.0')
    material.ThermalType('Normal')
    material.ThermalConductivity('401.0')
    material.HeatCapacity('0.39')
    material.MetabolicRate('0')
    material.BloodFlow('0')
    material.VoxelConvection('0')
    material.MechanicsType('Isotropic')
    material.YoungsModulus('120')
    material.PoissonsRatio('0.33')
    material.ThermalExpansionRate('17')
    material.Colour('1', '1', '0')
    material.Wireframe('False')
    material.Reflection('False')
    material.Allowoutline('True')
    material.Transparentoutline('False')
    material.Transparency('0')
    material.Create()
    format(material)
    
def copper_pure_lossy(mws):
    material = mws.Material

    material.Reset()
    material.Name('Copper (pure)')
    material.FrqType('all')  # warning
    material.Type('Lossy Metal')
    material.MaterialUnit('Frequency', 'GHz')
    material.MaterialUnit('Geometry', 'mm')
    material.MaterialUnit('Time', 's')
    material.MaterialUnit('Temperature', 'Kelvin')
    material.Mu('1.0')  # warning
    material.Sigma('5.96e+007')
    material.Rho('8930.0')
    material.ThermalType('Normal')
    material.ThermalConductivity('401.0')
    material.HeatCapacity('0.39')  # warning
    material.MetabolicRate('0')
    material.BloodFlow('0')
    material.VoxelConvection('0')
    material.MechanicsType('Isotropic')
    material.YoungsModulus('120')
    material.PoissonsRatio('0.33')
    material.ThermalExpansionRate('17')
    material.ReferenceCoordSystem('Global')
    material.CoordSystemType('Cartesian')
    material.NLAnisotropy('False')
    material.NLAStackingFactor('1')
    material.NLADirectionX('1')
    material.NLADirectionY('1')
    material.NLADirectionZ('1')
    material.ParticleProperty('SecondaryEmission')
    material.SeModel('Furman')
    material.SeVaughan('175', '2.25', '0', '1.0', '7.5')
    material.SePlot1D('True', '0', '100', '0', '1000')
    material.SeMaxSecondaries('10')
    material.SeTsParam_T1('0.66')
    material.SeTsParam_T2('0.80')
    material.SeTsParam_T3('0.70')
    material.SeTsParam_T4('1.00')
    material.SeTsParam_SEY('2.25')
    material.SeTsParam_Energy('175')
    material.SeTsParam_S('1.33')
    material.SeTsParam_PN('0', '1.6')
    material.SeTsParam_PN('1', '2.0')
    material.SeTsParam_PN('2', '1.8')
    material.SeTsParam_PN('3', '4.7')
    material.SeTsParam_PN('4', '1.8')
    material.SeTsParam_PN('5', '2.4')
    material.SeTsParam_PN('6', '1.8')
    material.SeTsParam_PN('7', '1.8')
    material.SeTsParam_PN('8', '2.3')
    material.SeTsParam_PN('9', '1.8')
    material.SeTsParam_EpsN('1', '6.20')
    material.SeTsParam_EpsN('2', '13.00')
    material.SeTsParam_EpsN('3', '8.80')
    material.SeTsParam_EpsN('4', '6.25')
    material.SeTsParam_EpsN('5', '2.25')
    material.SeTsParam_EpsN('6', '9.20')
    material.SeTsParam_EpsN('7', '5.30')
    material.SeTsParam_EpsN('8', '17.80')
    material.SeTsParam_EpsN('9', '10.00')
    material.SeRdParam_R('1.0')
    material.SeRdParam_R1('0.26')
    material.SeRdParam_R2('2.0')
    material.SeRdParam_Q('0.4')
    material.SeRdParam_P1Inf('0.01')
    material.SeRdParam_Energy('40.0')
    material.SeBsParam_Sigma('1.9')
    material.SeBsParam_E1('0.26')
    material.SeBsParam_E2('2.0')
    material.SeBsParam_P1Hat('0.02')
    material.SeBsParam_P1Inf('0.01')
    material.SeBsParam_Energy('0.0')
    material.SeBsParam_P('0.9')
    material.SeBsParam_W('100.0')
    material.FrqType('static')  # warning
    material.Type('Normal')
    material.MaterialUnit('Frequency', 'Hz')
    material.MaterialUnit('Geometry', 'mm')
    material.Epsilon('1')
    material.Mu('1.0')  # warning
    material.Kappa('5.96e+007')
    material.TanD('0')
    material.TanDFreq('0.0')
    material.TanDGiven('False')
    material.TanDModel('ConstTanD')
    material.KappaM('0.0')  # warning
    material.TanDM('0.0')
    material.TanDMFreq('0.0')
    material.TanDMGiven('False')
    material.TanDMModel('ConstTanD')
    material.DispModelEps('None')
    material.DispModelMu('None')
    material.DispersiveFittingSchemeEps('Nth Order')
    material.DispersiveFittingSchemeMu('Nth Order')
    material.UseGeneralDispersionEps('False')
    material.UseGeneralDispersionMu('False')
    material.Colour('1', '1', '0')
    material.Wireframe('False')
    material.Reflection('False')
    material.Allowoutline('True')
    material.Transparentoutline('False')
    material.Transparency('0')
    material.Create()
    format(material)

def fr4_lossy(mws):
    material = mws.material
    material.Reset()
    material.Name('FR-4 (lossy)')
    material.FrqType('all')
    material.Type('Normal')
    material.SetMaterialUnit('GHz', 'mm')
    material.Epsilon('4.3')
    material.Mue('1.0')
    material.Kappa('0.0')
    material.TanD('0.025')
    material.TanDFreq('10.0')
    material.TanDGiven('True')
    material.TanDModel('ConstTanD')
    material.KappaM('0.0')
    material.TanDM('0.0')
    material.TanDMFreq('0.0')
    material.TanDMModel('ConstKappa')
    material.DispModelEps('None')
    material.DispModelMue('None')
    material.DispersiveFittingSchemeEps('General 1st')
    material.DispersiveFittingSchemeMue('General 1st')
    material.UseGeneralDispersionEps('False')
    material.UseGeneralDispersionMue('False')
    material.Rho('0.0')
    material.ThermalType('Normal')
    material.ThermalConductivity('0.3')
    material.SetActiveMaterial('all')
    material.Colour('0.94', '0.82', '0.76')
    material.Wireframe('False')
    material.Transparency('0')
    material.Create()
    format(material)



# -*- coding: utf-8 -*-
"""
Created on Fri Jan 28 22:24:41 2022

@author: Yates


"""


def quit_project(mws):
    mws.Quit()

def save_project(mws):
    mws.Save()
    
def save_as_project(mws, filename):
    mws.SaveAs(f'{filename}.cst')

def save_zip(mws, keepAll, keep1D, keepFarfield, deleteProjFolder):
    mws.StoreinArchive(keepAll, keep1D, keepFarfield, deleteProjFolder)
    
def backup(mws, filename):
    mws.Backup(f'{filename}.cst')
    

    
# intialise default units 
def default_units(mws):
    units = mws.Units
    units.Geometry("mm")
    units.Frequency("GHz")
    units.TemperatureUnit("Kelvin")
    units.Time("ns")
    units.Voltage("V")
    units.Current("A")
    units.Resistance("Ohm")
    units.Conductance("Siemens")
    units.Capacitance("PikoF")
    units.Inductance("NanoH")
    

def activate_local_wcs(mws, setNormal, setOrigin, setUVector, activate):
    wcs = mws.WCS

    if activate:
        wcs.activateWCS('local')
        wcs.setNormal(str(setNormal(0)), str(setNormal(1)), str(setNormal(2)))
        wcs.setOrigin(str(setOrigin(0)), str(setOrigin(1)), str(setOrigin(2)))
        wcs.setUVector(str(setUVector(0)), str(setUVector(1)), str(setUVector(2)))
    else:
        wcs.ActivateWCS('Global')
    
def mesh_initiator(mws):
    FDSolver = mws.FDSolver
    mesh = mws.Mesh
    mesh_settings = mws.MeshSettings
    meshAdaption3D = mws.MeshAdaption3D
    PostProcess1D = mws.PostProcess1D

    FDSolver.ExtrudeOpenBC('True')

    mesh.MergeThinPECLayerFixpoints('True')
    mesh.RatioLimit('20')
    mesh.AutomeshRefineAtPecLines('True', '6')
    mesh.FPBAAvoidNonRegUnite('True')
    mesh.ConsiderSpaceForLowerMeshLimit('False')
    mesh.MinimumStepNumber('5')
    mesh.AnisotropicCurvatureRefinement('True')
    mesh.AnisotropicCurvatureRefinementFSM('True')
    
    # Defualt mesh settings 
    
    mesh_settings.SetMeshType('Hex')
    mesh_settings.Set('RatioLimitGeometry', '20')
    mesh_settings.Set('EdgeRefinementOn', '1')
    mesh_settings.Set('EdgeRefinementRatio', '6')
    mesh_settings.SetMeshType('HexTLM')
    mesh_settings.Set('RatioLimitGeometry', '20')
    mesh_settings.SetMeshType('Tet')
    mesh_settings.Set('VolMeshGradation', '1.5')
    mesh_settings.Set('SrfMeshGradation', '1.5')
    mesh_settings.SetMeshType('Hex')
    mesh_settings.Set('Version', '1%')

    meshAdaption3D.SetAdaptionStrategy('Energy')
    mesh.MeshType('PBA')
    PostProcess1D.ActivateOperation('vswr', 'true')
    PostProcess1D.ActivateOperation('yz-matrices', 'true')
    
def units(mws, geometry, frequency, time, temperatureUnit, voltage, current, resistance, conductance, capacitance, inductance):
    units = mws.Units

    units.Geometry(geometry)
    units.Frequency(frequency)
    units.TemperatureUnit(temperatureUnit)
    units.Time(time)
    units.Voltage(voltage)
    units.Current(current)
    units.Resistance(resistance)
    units.Conductance(conductance)
    units.Capacitance(capacitance)
    units.Inductance(inductance)
    
def background_material(mws, xmin, xmax, ymin, ymax, zmin, zmax):
    background = mws.Background
    material = mws.Material

    background.ResetBackground()
    background.Type('Normal')
    background.Epsilon('1.0')
    background.Mu('1.0')
    background.XminSpace(str(xmin))
    background.XmaxSpace(str(xmax))
    background.YminSpace(str(ymin))
    background.YmaxSpace(str(ymax))
    background.ZminSpace(str(zmin))
    background.ZmaxSpace(str(zmax))

    material.Reset()
    material.FrqType('all')
    material.Type('Normal')
    material.MaterialUnit('Frequency', 'Hz')
    material.MaterialUnit('Geometry', 'm')
    material.MaterialUnit('Time', 's')
    material.MaterialUnit('Temperature', 'Kelvin')
    material.Epsilon('1.0')
    material.Mue('1.0')
    material.Sigma('0.0')
    material.TanD('0.0')
    material.TanDFreq('0.0')
    material.TanDGiven('False')
    material.TanDModel('ConstSigma')
    material.EnableUserConstTanDModelOrderEps('False')
    material.ConstTanDModelOrderEps('1')
    material.SetElParametricConductivity('False')
    material.ReferenceCoordSystem('Global')
    material.CoordSystemType('Cartesian')
    material.SigmaM('0')
    material.TanDM('0.0')
    material.TanDMFreq('0.0')
    material.TanDMGiven('False')
    material.TanDMModel('ConstSigma')
    material.EnableUserConstTanDModelOrderMue('False')
    material.ConstTanDModelOrderMue('1')
    material.SetMagParametricConductivity('False')
    material.DispModelEps('None')
    material.DispModelMue('None')
    material.DispersiveFittingSchemeEps('Nth Order')
    material.MaximalOrderNthModelFitEps('10')
    material.ErrorLimitNthModelFitEps('0.1')
    material.UseOnlyDataInSimFreqRangeNthModelEps('False')
    material.DispersiveFittingSchemeMue('Nth Order')
    material.MaximalOrderNthModelFitMue('10')
    material.ErrorLimitNthModelFitMue('0.1')
    material.UseOnlyDataInSimFreqRangeNthModelMue('False')
    material.UseGeneralDispersionEps('False')
    material.UseGeneralDispersionMue('False')
    material.NLAnisotropy('False')
    material.NLAStackingFactor('1')
    material.NLADirectionX('1')
    material.NLADirectionY('0')
    material.NLADirectionZ('0')
    material.Rho('0.0')
    material.ThermalType('Normal')
    material.ThermalConductivity('0.0')
    material.HeatCapacity('0.0')
    material.MetabolicRate('0')
    material.BloodFlow('0')
    material.VoxelConvection('0')
    material.MechanicsType('Unused')
    material.Colour('0.6', '0.6', '0.6')
    material.Wireframe('False')
    material.Reflection('False')
    material.Allowoutline('True')
    material.Transparentoutline('False')
    material.Transparency('0')
    material.ChangeBackgroundMaterial()
    
def mesh_settings(mws, cellsPerWavelength, minCell):
    mesh = mws.Mesh
    mesh_settings = mws.MeshSettings
    discretiser = mws.Discretizer

    mesh.MeshType('PBA')
    mesh.SetCreator('High Frequency')  # warning

    mesh_settings.SetMeshType('Hex')
    mesh_settings.Set('Version', '1%')
    mesh_settings.Set.StepsPerWaveNear(str(cellsPerWavelength))
    mesh_settings.Set.StepsPerWaveFar(str(cellsPerWavelength))
    mesh_settings.Set.WavelengthRefinementSameAsNear('1')
    mesh_settings.Set.StepsPerBoxNear(str(cellsPerWavelength))
    mesh_settings.Set.StepsPerBoxFar(str(cellsPerWavelength))
    mesh_settings.Set.MaxStepNear(str(cellsPerWavelength))
    mesh_settings.Set.MaxStepFar(str(cellsPerWavelength))
    mesh_settings.Set.ModelBoxDescrNear('maxedge')
    mesh_settings.Set.ModelBoxDescrFar('maxedge')
    mesh_settings.Set.UseMaxStepAbsolute('0')
    mesh_settings.Set.GeometryRefinementSameAsNear('1')
    mesh_settings.Set.UseRatioLimitGeometry('1')
    mesh_settings.Set.RatioLimitGeometry(str(minCell))
    mesh_settings.Set.MinStepGeometryX('0')
    mesh_settings.Set.MinStepGeometryY('0')
    mesh_settings.Set.MinStepGeometryZ('0')
    mesh_settings.Set.UseSameMinStepGeometryXYZ('1')
    mesh_settings.SetMeshType('Hex')
    mesh_settings.Set.FaceRefinementOn('0')
    mesh_settings.Set.FaceRefinementPolicy('2')
    mesh_settings.Set.FaceRefinementRatio('2')
    mesh_settings.Set.FaceRefinementStep('0')
    mesh_settings.Set.FaceRefinementNSteps('2')
    mesh_settings.Set.EllipseRefinementOn('0')
    mesh_settings.Set.EllipseRefinementPolicy('2')
    mesh_settings.Set.EllipseRefinementRatio('2')
    mesh_settings.Set.EllipseRefinementStep('0')
    mesh_settings.Set.EllipseRefinementNSteps('2')
    mesh_settings.Set.FaceRefinementBufferLines('3')
    mesh_settings.Set.EdgeRefinementOn('1')
    mesh_settings.Set.EdgeRefinementPolicy('1')
    mesh_settings.Set.EdgeRefinementRatio('2')
    mesh_settings.Set.EdgeRefinementStep('0')
    mesh_settings.Set.EdgeRefinementBufferLines('3')
    mesh_settings.Set.RefineEdgeMaterialGlobal('0')
    mesh_settings.Set.RefineAxialEdgeGlobal('0')
    mesh_settings.Set.BufferLinesNear('3')
    mesh_settings.Set.UseDielectrics('1')
    mesh_settings.Set.EquilibrateOn('0')
    mesh_settings.Set.Equilibrate('1.5')
    mesh_settings.Set.IgnoreThinPanelMaterial('0')
    mesh_settings.SetMeshType('Hex')
    mesh_settings.Set.SnapToAxialEdges('1')
    mesh_settings.Set.SnapToPlanes('1')
    mesh_settings.Set.SnapToSpheres('1')
    mesh_settings.Set.SnapToEllipses('1')
    mesh_settings.Set.SnapToCylinders('1')
    mesh_settings.Set.SnapToCylinderCenters('1')
    mesh_settings.Set.SnapToEllipseCenters('1')

    discretiser.MeshType('PBA')
    discretiser.PBAType('Fast PBA')
    discretiser.AutomaticPBAType('True')
    discretiser.FPBAAccuracyEnhancement('enable')
    discretiser.ConnectivityCheck('False')
    discretiser.ConvertGeometryDataAfterMeshing('True')
    discretiser.UsePecEdgeModel('True')
    discretiser.GapDetection('False')
    discretiser.FPBAGapTolerance('1e-3')
    discretiser.SetMaxParallelMesherThreads('Hex', '12')
    discretiser.SetParallelMesherMode('Hex', 'Maximum')
    discretiser.PointAccEnhancement('0')
    discretiser.UseSplitComponents('True')
    discretiser.EnableSubgridding('False')
    discretiser.PBAFillLimit('99')
    discretiser.AlwaysExcludePec('False')

    




def efield_monitor(mws, eFieldName, frequency):
    '''
    

    Parameters
    ----------
    mws : TYPE
        DESCRIPTION.
    eFieldName : TYPE
        DESCRIPTION.
    frequency : TYPE
        DESCRIPTION.

    Returns
    -------
    None.

    '''
    
    monitor = mws.Monitor
    monitor.Reset()
    monitor.Name(eFieldName)
    monitor.Dimension('Volume')
    monitor.Domain('Frequency')
    monitor.FieldType('Efield')
    monitor.Frequency(str(frequency))
    monitor.UseSubvolume('False')
    monitor.SetSubvolume('-53.310273111111', '53.310273111111', '-53.310273111111', '53.310273111111',
                         '-33.310273111111', '71.310273111111')
    monitor.Create()
    
def hfield_monitor(mws, hFieldName, frequency):
    
    monitor = mws.Monitor
    monitor.Reset()
    monitor.Name(hFieldName)
    monitor.Dimension('Volume')
    monitor.Domain('Frequency')
    monitor.FieldType('Hfield')
    monitor.Frequency(str(frequency))
    monitor.UseSubvolume('False')
    monitor.SetSubvolume('-209.896229', '229.896229', '-179.896229', '179.896229', '-149.896229', '187.896229')
    monitor.Create()


def farfield_monitor(mws, farfieldName, frequency):
    
    monitor = mws.Monitor
    monitor.Reset()
    monitor.Name(farfieldName)
    monitor.Domain('Frequency')
    monitor.FieldType('Farfield')
    monitor.Frequency(str(frequency))
    monitor.UseSubvolume('False')
    monitor.ExportFarfieldSource('False')
    monitor.SetSubvolume('-53.310273111111', '53.310273111111', '-53.310273111111', '53.310273111111',
                         '-33.310273111111', '71.310273111111')
    monitor.Create()
    
##############################################################################

def frequency_range(mws, frange1, frange2):
    
    Solver = mws.Solver
    Solver.FrequencyRange(str(frange1), str(frange2))

    mesh = mws.Mesh
    mesh_settings = mws.MeshSettings
    mesh_settings.SetMeshType('Hex')
    mesh_settings.Set('Version', '1%')
    mesh.MeshType('PBA')

def waveguide_port(mws, portNumber, xrange, yrange, zrange, xrangeAdd, yrangeAdd, zrangeAdd, coordinates, orientation):
    
    port = mws.Port
    port.Reset()
    port.PortNumber(str(portNumber))
    port.Label('')
    port.NumberOfModes('1')
    port.AdjustPolarization('False')
    port.PolarizationAngle('0.0')
    port.ReferencePlaneDistance('0')
    port.TextSize('50')
    port.Coordinates(coordinates)
    port.Orientation(orientation)
    port.PortOnBound('False')
    port.ClipPickedPortToBound('False')
    port.Xrange(str(xrange[0]), str(xrange[1]))
    port.Yrange(str(yrange[0]), str(yrange[1]))
    port.Zrange(str(zrange[0]), str(zrange[1]))
    port.XrangeAdd(str(xrangeAdd[0]), str(xrangeAdd[1]))
    port.YrangeAdd(str(yrangeAdd[0]), str(yrangeAdd[1]))
    port.ZrangeAdd(str(zrangeAdd[0]), str(zrangeAdd[1]))
    port.SingleEnded('False')
    port.Create()

    
def plane_wave_excitation(mws, normal, eVector, polarization, referenceFrequency, phaseDifference, circularDirection, axialRatio):
    
    planeWave = mws.PlaneWave
    planeWave.Reset()
    planeWave.Normal(str(normal(0)), str(normal(1)), str(normal(2)))
    planeWave.EVector(str(eVector(0)), str(eVector(1)), str(eVector(2)))
    planeWave.Polarization(polarization)
    planeWave.ReferenceFrequency(str(referenceFrequency))
    planeWave.PhaseDifference(str(phaseDifference))
    planeWave.CircularDirection(circularDirection)
    planeWave.AxialRatio(str(axialRatio))
    planeWave.SetUserDecouplingPlane('False')
    planeWave.Store()

    
def discrete_port(mws, portNumber, setP1, setP2):
    
    discrete_port = mws.DiscretePort
    discrete_port.Reset()
    discrete_port.PortNumber(str(portNumber))
    discrete_port.Type('SParameter')
    discrete_port.Label('')
    discrete_port.Impedance('50')
    discrete_port.VoltagePortImpedance('0.0')
    discrete_port.Voltage('1.0')
    discrete_port.Current('1.0')
    discrete_port.SetP1('False', setP1[0], setP1[1], setP1[2])
    discrete_port.SetP2('False', setP2[0], setP2[1], setP2[2])
    discrete_port.InvertDirection('False')
    discrete_port.LocalCoordinates('False')
    discrete_port.Monitor('True')
    discrete_port.Radius('0.0')
    discrete_port.Wire('')
    discrete_port.Position('end1')
    discrete_port.Create()
    format(discrete_port)

    
def discrete_face_port(mws, portNumber, setP1, setP2):
    
    discrete_face_port = mws.DiscreteFacePort
    discrete_face_port.Reset()
    discrete_face_port.PortNumber(portNumber)
    discrete_face_port.Type('SParameter')
    discrete_face_port.Label('')
    discrete_face_port.Impedance('50')
    discrete_face_port.VoltagePortImpedance('0.0')
    discrete_face_port.VoltageAmplitude('1.0')
    discrete_face_port.setP1('True', setP1[0], setP1[1], setP1[2])
    discrete_face_port.setP2('True', setP2[0], setP2[1], setP2[2])
    discrete_face_port.InvertDirection('False')
    discrete_face_port.LocalCoordinates('False')
    discrete_face_port.Monitor('True')
    discrete_face_port.CenterEdge('True')
    discrete_face_port.UseProjection('False')
    discrete_face_port.ReverseProjection('False')
    discrete_face_port.Create()
    format(discrete_face_port)

    
def time_domain_solver(mws, steadyStateLimit):
    
    mesh = mws.Mesh
    mesh.SetCreator('High Frequency')
    
    solver = mws.Solver
    solver.Method('Hexahedral')
    solver.CalculationType('TD-S')
    solver.StimulationPort('All')
    solver.StimulationMode('All')
    solver.SteadyStateLimit(str(steadyStateLimit))
    solver.MeshAdaption('False')
    solver.NormingImpedance('50')
    solver.CalculateModesOnly('False')
    solver.SParaSymmetry('False')
    solver.StoreTDResultsInCache('False')
    solver.FullDeembedding('False')
    solver.SuperimposePLWExcitation('False')
    solver.UseSensitivityAnalysis('False')
    solver.MaximumNumberOfThreads('12')
    solver.Start


def open_boundary(mws, minfrequency, xmin, xmax, ymin, ymax, zmin, zmax):
    boundary = mws.Boundary
    plot = mws.Plot

    boundary.Xmin(xmin)
    boundary.Xmax(xmax)
    boundary.Ymin(ymin)
    boundary.Ymax(ymax)
    boundary.Zmin(zmin)
    boundary.Zmax(zmax)
    boundary.Xsymmetry('none')
    boundary.Ysymmetry('none')
    boundary.Zsymmetry('none')
    boundary.XminThermal('isothermal')
    boundary.XmaxThermal('isothermal')
    boundary.YminThermal('isothermal')
    boundary.YmaxThermal('isothermal')
    boundary.ZminThermal('isothermal')
    boundary.ZmaxThermal('isothermal')
    boundary.XsymmetryThermal('none')
    boundary.YsymmetryThermal('none')
    boundary.ZsymmetryThermal('none')
    boundary.ApplyInAllDirections('False')
    boundary.ApplyInAllDirectionsThermal('False')
    boundary.XminTemperature('')
    boundary.XminTemperatureType('None')
    boundary.XmaxTemperature('')
    boundary.XmaxTemperatureType('None')
    boundary.YminTemperature('')
    boundary.YminTemperatureType('None')
    boundary.YmaxTemperature('')
    boundary.YmaxTemperatureType('None')
    boundary.ZminTemperature('')
    boundary.ZminTemperatureType('None')
    boundary.ZmaxTemperature('')
    boundary.ZmaxTemperatureType('None')
    if xmin == 'unit cell':
        boundary.XPeriodicShift('0.0')
        boundary.YPeriodicShift('0.0')
        boundary.ZPeriodicShift('0.0')
        boundary.PeriodicUseConstantAngles('False')
        boundary.SetPeriodicBoundaryAngles('0.0', '0.0')
        boundary.SetPeriodicBoundaryAnglesDirection('outward')
        boundary.UnitCellFitToBoundingBox('True')
        boundary.UnitCellDs1('0.0')
        boundary.UnitCellDs2('0.0')
        boundary.UnitCellAngle('90.0')
    if xmin == 'expanded open':
        boundary.ReflectionLevel('0.0001')
        boundary.MinimumDistanceType('Fraction')
        boundary.MinimumDistancePerWavelengthNewMeshEngine('4')
        boundary.MinimumDistanceReferenceFrequencyType('CenterNMonitors')
        boundary.FrequencyForMinimumDistance(str(minfrequency))
        boundary.SetAbsoluteDistance('0.0')
        plot.DrawBox('True')



# -*- coding: utf-8 -*-
"""
Created on Fri Jan 21 20:40:37 2022

@author: Will

Provides modelling functionality 
"""
# Modelling options
##############################################################################
def add(mws, component1, compenent2):
    
    solid = mws.Solid
    solid.Add(component1, compenent2)
    
def subtract(mws, component1, component2):
    
    solid = mws.Solid
    solid.Subtract(component1, component2)
    
def align_wcs_with_face(mws):
    
    wcs = mws.WCS
    wcs.AlignWCSWithSelected('Face')

def align_wcs_with_point(mws):
    
    wcs = mws.WCS
    wcs.AlignWCSWithSelected('Point')

def clear_picks(mws):
    
    pick = mws.Pick
    pick.ClearAllPicks()
    
def pick_edge(mws, componentName, position):
    
    pick = mws.Pick
    pick.PickEdgeFromId((f'component1:{componentName}'), str(position), str(position))

def pick_face(mws, name, _id):
    
    pick = mws.Pick
    pick.PickFaceFromId((f'component1:{name}'), str(_id))   
    
def pick_face_from_point(mws, component, name, xPoint, yPoint, zPoint):
    
    pick = mws.Pick
    pick.PickFaceFromPoint((f'component1:{name}'), str(xPoint), str(yPoint), str(zPoint))
    
def pick_mid_point(mws, name, _id):
    
    pick = mws.Pick
    pick.PickMidpointFromId((f'component1:{name}'), str(_id))

    
# Shapes
##############################################################################
def brick(mws, Name, component, material, xrange, yrange, zrange):
    
    brick = mws.Brick
    brick.Reset()
    brick.Name(Name)
    brick.component(component)
    brick.Material(material)
    brick.xrange(str(xrange[0]), str(xrange[1]))
    brick.yrange(str(yrange[0]), str(yrange[1]))
    brick.zrange(str(zrange[0]), str(zrange[1]))
    brick.Create
    format(brick)

def cylinder(mws, Name, component, material, Axis, OuterRadius, InnerRadius, Xcenter, Ycenter, Zrange):
    
    cylinder = mws.Cylinder
    cylinder.Reset()
    cylinder.Name(Name)
    cylinder.Component(component)
    cylinder.Material(material)
    cylinder.Axis(Axis)
    cylinder.Outerradius(OuterRadius)
    cylinder.Innerradius(InnerRadius)

    if Axis == 'Z':
        cylinder.Xcenter(str(Xcenter))
        cylinder.Ycenter(str(Ycenter))
        cylinder.Zrange(str(Zrange[0]), str(Zrange[1]))
    elif Axis == 'X':
        cylinder.Ycenter(str(Ycenter))
        cylinder.Zcenter(str(Xcenter))
        cylinder.Xrange(str(Zrange[0]), str(Zrange[1]))
    elif Axis == 'Y':
        cylinder.Xcenter(str(Xcenter))
        cylinder.Zcenter(str(Ycenter))
        cylinder.Yrange(str(Zrange[0]), str(Zrange[1]))

    cylinder.Create()
    format(cylinder)

def sphere(mws, name, component, material, axis, centreRadius, topRadius, bottomRadius, centre):
    
    sphere = mws.Sphere
    sphere.Reset()
    sphere.Name(name)
    sphere.Component(component)
    sphere.Material(material)
    sphere.Axis(axis)
    sphere.CenterRadius(str(centreRadius))
    sphere.TopRadius(str(topRadius))
    sphere.BottomRadius(str(bottomRadius))
    sphere.Centre(str(centre(0)), str(centre(1)), str(centre(2)))
    sphere.Segments('0')
    sphere.Create()



# -*- coding: utf-8 -*-
"""
Created on Sun Feb 13 17:05:45 2022

@author: mario
"""
from .session import *
from .model import * 
from .material import *
from .home import * 
from .post_process import *
from .simulation import *


'''
Post processing functions for CST 2021

TODO:
    Include more result types from the 1D tree. 
    Include 2D/3D and Farfield result trees. 
'''

import math

def result_parameters(mws, *, parent_path=r'1D Results\S-Parameters', run_id=0, result_id=0):
    '''
    Provides access to S-parameters from the 1D results tree.

    Parameters
    ----------
    mws : TYPE
        DESCRIPTION.
    * : TYPE
        DESCRIPTION.
    parent_path : TYPE, optional
        DESCRIPTION. The default is r'1D Results\S-Parameters'.
    run_id : TYPE, optional
        DESCRIPTION. The default is 0.
    result_id : TYPE, optional
        DESCRIPTION. The default is 0.

    Returns
    -------
    TYPE
        DESCRIPTION.

    '''
    
    result_tree = mws.Resulttree
    
    result_path_list = list()
    if result_tree.DoesTreeItemExist(parent_path):
        child = result_tree.GetFirstChildName(parent_path)

        while len(child) > 0:
            result_path_list.append(child)
            child = result_tree.GetNextItemName(child)

        run_ids = result_tree.GetResultIDsFromTreeItem(result_path_list[result_id])
        run_id_name = list(run_ids)[run_id]
        object_res = result_tree.GetResultFromTreeItem(result_path_list[result_id], run_id_name)
        result_type = object_res.GetResultObjectType

        frequencies_list = list(object_res.GetArray('x'))

        if result_type == '1DC':
            y_real = list(object_res.GetArray('yre'))
            y_imag = list(object_res.GetArray('yim'))

            y_list = []
            for i, yval in enumerate(y_real):
                y_list.append(20 * math.log10(abs(complex(y_real[i], y_imag[i]))))
        else:
            y_list = list(object_res.GetArray('y'))

        x_label = object_res.GetXLabel
        y_label = object_res.GetYLabel
        plot_title = object_res.GetTitle

        if result_type == '1DC':
            return frequencies_list, [y_real, y_imag], y_list, [x_label, y_label, plot_title]
        else:
            return frequencies_list, y_list, [x_label, y_label, plot_title]
    else:
        print('Result tree item not found.')
		
def export_touchstone(mws, exportpath):

    touchstone = mws.TOUCHSTONE
    touchstone.Reset()
    touchstone.FileName(exportpath)
    touchstone.Impedance('50')
    touchstone.FrequencyRange('full')
    touchstone.Renormalize('True')
    touchstone.UseARResults('False')
    touchstone.SetNSamples('1001')
    touchstone.Write()
	



# -*- coding: utf-8 -*-
"""
Created on Sun Feb 13 17:15:38 2022

@author: Yates
"""
try:
    import win32com.client
except ModuleNotFoundError:
    print("Cheap fix for readthedocs")
    pass

def start():
    """
    Loads a local CST session.
    
    Since this is a Window-specific API, this functions creates an instance of
    a stand-alone client.
    
    TODO:
        In the far, far future, the need to open a Circuits & Systems or PCB
        instance may be required. I should look into that at some point. 
    """
    
    cst = win32com.client.dynamic.Dispatch("CSTStudio.Application")
    new_mws = cst.NewMWS()
    mws = cst.Active3D()
    
    return mws


def close():
    
    cst = win32com.client.dynamic.Dispatch("CSTStudio.Application")
    print("Exiting...")
    cst.quit()



try:
        from setuptools import setup
except ImportError:
        from distutils.core import setup
import sys

extra = {}
# if sys.version_info >= (3, ):
#    extra['use_2to3'] = True

setup(
    name='PyCST',
    packages=['pycst', ],
    version='0.0.1',
    author='William Yates',
    author_email='william.yates4@gmail.com',
    url='https://github.com/Yates1011/PyCST',
    description='Python interface to the CST',
    long_description='''
    PyCST''',
    classifiers=['Development Status :: 3 - Alpha',
                 'License :: OSI Approved :: Apache Software License',
                 'Natural Language :: English',
                 'Intended Audience :: Developers',
                 'Topic :: Scientific/Engineering :: Mathematics',
                 'Operating System :: OS Independent',
                 'Programming Language :: Python'],
    install_requires=['requests'],
    **extra
    )
